//------------------------------------------------------------------------------
// Paul Dibden 
// BA(HONS)Computer Visualisation & Animation 2004
// Innovations - 3D Pen and Ink Illustration shader
//------------------------------------------------------------------------------

surface peninkillustration(
	float widthofink = 0.2, levelofdetail = 1, outline = 0.2;
	
//adjusting 'levelofdetail' will vary the amount of lines drawn on the object,
//obviously the further from the camera the less detail will be needed.
	
	color blackpen = color(0,0,0);

	float ink1 = 1, ink2 = 1, ink3 = 1;

//if ink values == 1, then lines will be drawn from its direction plane,
//any other value and the lines will not be drawn. This allows 1, 2 or 3
//sets of lines at varied angles to be drawn.

	float  clipvalue1 = 0.8, clipvalue2 = 0.6, clipvalue3 = 0.7;

//line clipping values for each of the line direction planes.

	float  taperinitialise1 = 0.3, taperinitialise2 = 0.3, taperinitialise3 = 0.3;

//tapering values for each of the line direction planes.

      float scale = 1;	

//scales all the boundary parameters into range for detail.

	float dir1x = 0, dir1y = 2, dir1z = 3;
//direction plane 1, specified by its normal in world space.
	float dir2x = 0, dir2y = 3, dir2z = -2;
//direction plane 2, specified by its normal in world space.
	float dir3x = 0, dir3y = 4, dir3z = 1;
//direction plane 3, specified by its normal in world space.

	float Ka = 0.2, Kd = 0.8, Ks = 0.6, roughness = 0.1;

//parameters to the shader, all default values.
	)

{
	point PP;
	float x, y, z;
	float dist, detail, width;
	float a, b, c;
	float illumination;
	float which, clip, taper;

	normal Nf = faceforward(normalize(N), I);	//variable N is surface normal.
	vector V = -normalize(I);	//normalised 'view'.

//compute the forward-facing normal Nf and the vector towards the ray origin V,
//both are normalised. These vectors will be needed by both 'diffuse' and 'specular'
//for the arguments to the lighting function.

	PP =  transform("shader", P);
	PP = PP+point noise(PP*10)*0.02;	//adding noise to the point(PP).

//Each point to be shaded by its position in 3D "shader" coordinate space.
//The position(P)is transformed into shader space, and then assigned to a
//new variable(PP). Calculations will now be done relative to the position
//of the shader, and the surface colour calculation will be based upon this
//value.

	x = xcomp(PP);
	y = ycomp(PP);
	z = zcomp(PP);

//The components of the point(PP) are extracted using the functions xcomp,
//ycomp and zcomp.

	illumination = comp( Oi * color(1,1,1) * (Ka * ambient() + Kd * diffuse(Nf)) + Ks
			     * specular(Nf, V, roughness), 0);

//illumination at this point is then calculated,and the value attached to 'illumination'.

	Oi = Os;
	Ci = color(1,1,1);	

/*--------------------- Pen and Ink Direction 1---------------------*/

if (ink1 == 1)
{
	a = dir1x;
	b = dir1y;
	c = dir1z;
	
	dist = abs(a*x+b*y+c*z)/sqrt(a*a+b*b+c*c);

//the above is the algorithm used to solve the distance of a point from a plane,
//basically acts as a 'ray query' for my shader. So for every point that is
//shaded, its distance from the direction planes needs to be solved.
//In this case direction plane 1's distance is being solved.

	detail = levelofdetail * 28;
	dist *= detail;

//distance is then multiplied by the 'detail' value.

	which = mod(floor(dist), 2);

//mod()divides the 1st no. by the 2nd no. then returns the remainder,
//this determines the strip pattern. Using floor() will in this case
//give mod() the integer part of the value, this then in turn will
//give 'which' the remainder. 

	dist -= floor(dist);	

	clip =  scale * clipvalue1 * pow(0.5, which);
	taper = scale * taperinitialise1 * pow(0.5, which);
	width = widthofink;
	width = width * (1-smoothstep(clip-taper/2, clip+taper/2, illumination));

//varying the line width according to the value of illumination.
//smoothstep() takes the two parameters that represent the start and end of
//a transition, while the third parameter represents the value to be tested.

	dist = smoothstep( 0, width*0.25, dist) - smoothstep( width, width*1.25, dist);
	Ci = mix( Ci, blackpen, dist);	//shader output

//mix()takes the two colours and blends them together based upon the third.

}

/*--------------------- Pen and Ink Direction 2---------------------*/

if (ink2 == 1)
{
	a = dir2x;
	b = dir2y;
	c = dir2z;
	
	dist = abs(a*x+b*y+c*z)/sqrt(a*a+b*b+c*c);

//abs() to remove the sign of the result.

	detail = levelofdetail * 28;
	dist *= detail;
	which = mod(floor(dist), 2);
	dist -= floor(dist);

	clip =  scale * clipvalue2 * pow(0.5, which);
	taper = scale * taperinitialise2 * pow(0.5, which);

//clip and taper values are halved on every other line.

	width = widthofink;
	width = width * (1-smoothstep(clip-taper/2, clip+taper/2, illumination));

	dist = smoothstep( 0, width*0.25, dist) - smoothstep( width, width*1.25, dist);
	Ci = mix( Ci, blackpen, dist);
}

/*--------------------- Pen and Ink Direction 3---------------------*/

if (ink3 == 1)
{
	a = dir3x;
	b = dir3y;
	c = dir3z;
	
	dist = abs(a*x+b*y+c*z)/sqrt(a*a+b*b+c*c);

	detail = levelofdetail * 28;
	dist *= detail;
	which = mod(floor(dist), 2);
	dist -= floor(dist);

//floor() obtains integer part.

	clip =  scale * clipvalue3 * pow(0.5, which);
	taper = scale * taperinitialise3 * pow(0.5, which);
	width = widthofink;
	width = width * (1-smoothstep(clip-taper/2, clip+taper/2, illumination));

//the above holds or decreases the width of the line depending on illumination intensity.
//smoothstep() function returns 0 if x < edge 0, returns 1 if x > edge 1, and performs
//a smooth hermite interpolation between 0 and 1 when edge 0 < x < edge 1.
//This is useful here for a thresholding with a smooth transition.

	dist = smoothstep( 0, width*0.25, dist) - smoothstep( width, width*1.25, dist);

//the resulting variable is then used to determine whether the point
//is black, white or a blend of the two.

	Ci = mix( Ci, blackpen, dist);
}

	float edge = smoothstep(outline - 0.05, outline, Nf.V);

//to work out the silhouette edge with V as the eye vector and Nf as a surface normal,
//a silhouette is defined as the point on the surface when V.Nf.

	Ci = mix( blackpen, Ci, edge);
}





	
	
	


		
	

	

	






